#!/usr/bin/perl -w
#
# ndshow - Prints out values of interesting network driver parameters.
# 
# Copyright (c) 2001-2005 Occam's Razor. All rights reserved.
#
# See the LICENSE file distributed with this code for restrictions on its use
# and further distribution.
# Original distribution available at <http://www.occam.com/tools/>.
#
# $Id: ndshow.pl,v 1.12 2008/05/10 17:27:32 leonvs Exp $
#
# TODO
#	confirm operation on all platforms, port to others
#	expand list of parameters (esp. from Linux, AIX, BSD, ...)?
#	set up hashes in a database, or at least flat file(s)?
#	time units are currently all displayed in seconds, memory in bytes; is
#		that wise, or should we automatically convert to the largest
#		sensible unit?
#	use Config module to get OS
#	reduce duplication of lines for different OS versions
#

use strict;

# global variables
my $progname;
my $osname;
my $osver;
my @parameter = ();
my %location = ();
my %type = ();
my $show;

setup();
showParams();

sub setup {
	$progname = $0; $progname =~ s/.*\///;
	my $USAGE = <<EOF;
usage:	$progname

	Pretty simple, really.
EOF

	@ARGV and die $USAGE;

	($osname, $osver) = split /\s+/, getSystem();

	setupParamList();
	setupParamHashes();
}

# getSystem returns OS name and version.
sub getSystem {
	my $uname = "";
	# This is kind of lame.
	(-x "/sbin/uname" and $uname = "/sbin/uname") or
		(-x "/bin/uname" and $uname = "/bin/uname") or
		(-x "/usr/bin/uname" and $uname = "/usr/bin/uname");
	return `$uname -sr`;
}

sub setupParamList {
	@parameter = (
		"IP Packet Forwarding",
		"IP Directed Broadcast Packet Forwarding",
		"IP Source-routed Packet Forwarding",
		"IP Source-routed Packet Receiving",
		"IP Source-routed Packet Sending",
		"IP Source-routed Packet Non-local Sending",
		"IP Strict Multihoming",
		"IP Path MTU Discovery",
		"IP Local Port Range",
		"IPv6 Packet Forwarding",
		"IPv6 Source-routed Packet Forwarding",
		"IPv6 Strict Multihoming",
		"ICMP Echo Broadcast Receiving",
		"ICMP Timestamp Receiving",
		"ICMP Timestamp Broadcast Receiving",
		"ICMP Redirect Receiving",
		"ICMP Redirect Sending",
		"ICMPv6 Echo Multicast Receiving",
		"ICMPv6 Redirect Receiving",
		"ICMPv6 Redirect Sending",
		"TCP Incomplete Connection Queue Size",
		"TCP Completed Connection Queue Size",
		"TCP Connection Timeout",
		"TCP Idle Timeout",
		"TCP Keepalive Interval",
		"TCP TIME_WAIT Timeout",
		"TCP FIN_WAIT_2 Timeout",
		"TCP FIN_WAIT_2 Keepalive Interval",
		"TCP ACK Delay",
		"TCP ACK Delay Buffer Size",
		"TCP ACK Delay Queue Size",
		"TCP ACK Delay LAN Buffer Size",
		"TCP ACK Delay LAN Queue Size",
		"TCP Nagle Algorithm Buffer Size",
		"TCP Slow Start Mode",
		"TCP Slow Start After Idle Mode",
		"TCP SYN Flood Protection",
		"TCP Sequence Number Generation Mode",
		"TCP Blind Attack Protection Mode",
		"TCP RST Rate Limit Mode",
		"TCP RST Rate Limit",
		"UDP ICMP Error Rate Limit Mode",
		"UDP ICMP Error Rate Limit",
		"TCP Extra Privileged Ports",
		"UDP Extra Privileged Ports",
		"TCP Path MTU Discovery",
		"UDP Path MTU Discovery",
	);
}

sub setupParamHashes {
	# %location
	#	key: parameter name
	#	value: argument to $show command
	%location = ();

	# %type
	#	key: parameter name
	#	value: one of the following, describing return value:
	#		boolean
	#		antiboolean ("0" means parameter is "on")
	#		integer
	#		byte
	#		kilobyte
	#		block (512 KB)
	#		megabyte
	#		millisecond
	#		halfsecond (AIX)
	#		perhalfsecond (AIX)
	#		second
	#		persecond
	#		quarterminute (AIX sure is quirky)
	#		minute
	#		string
	%type = ();

	# Solaris
	if ($osname eq "SunOS" && $osver =~ /^5\./) {
		$show = "/usr/sbin/ndd";

		$location{"IP Packet Forwarding"} = "/dev/ip ip_forwarding";
		$location{"IP Directed Broadcast Packet Forwarding"} = "/dev/ip ip_forward_directed_broadcasts";
		$location{"IP Source-routed Packet Forwarding"} = "/dev/ip ip_forward_src_routed";
		$location{"IP Strict Multihoming"} = "/dev/ip ip_strict_dst_multihoming";
		$location{"IP Path MTU Discovery"} = "/dev/ip ip_path_mtu_discovery";
		$location{"ICMP Echo Broadcast Receiving"} = "/dev/ip ip_respond_to_echo_broadcast";
		$location{"ICMP Timestamp Receiving"} = "/dev/ip ip_respond_to_timestamp";
		$location{"ICMP Timestamp Broadcast Receiving"} = "/dev/ip ip_respond_to_timestamp_broadcast";
		$location{"ICMP Redirect Receiving"} = "/dev/ip ip_ignore_redirect";
		$location{"ICMP Redirect Sending"} = "/dev/ip ip_send_redirects";
		$location{"TCP Completed Connection Queue Size"} = "/dev/tcp tcp_conn_req_max";
		$location{"TCP Connection Timeout"} = "/dev/tcp tcp_ip_abort_cinterval";
		$location{"TCP Idle Timeout"} = "/dev/tcp tcp_ip_abort_interval";
		$location{"TCP Keepalive Interval"} = "/dev/tcp tcp_keepalive_interval";
		$location{"TCP TIME_WAIT Timeout"} = "/dev/tcp tcp_close_wait_interval";
		$location{"TCP FIN_WAIT_2 Timeout"} = "/dev/tcp tcp_fin_wait_2_flush_interval";
		$location{"TCP ACK Delay Buffer Size"} = "/dev/tcp tcp_deferred_ack_interval";
		$location{"TCP Nagle Algorithm Buffer Size"} = "/dev/tcp tcp_naglim_def";
		$location{"TCP Sequence Number Generation Mode"} = "/dev/tcp tcp_strong_iss";
		$location{"TCP Extra Privileged Ports"} = "/dev/tcp tcp_extra_priv_ports";
		$location{"UDP Extra Privileged Ports"} = "/dev/udp udp_extra_priv_ports";

		$type{"IP Packet Forwarding"} = "boolean";
		$type{"IP Directed Broadcast Packet Forwarding"} = "boolean";
		$type{"IP Source-routed Packet Forwarding"} = "boolean";
		$type{"IP Strict Multihoming"} = "boolean";
		$type{"IP Path MTU Discovery"} = "boolean";
		$type{"ICMP Echo Broadcast Receiving"} = "boolean";
		$type{"ICMP Timestamp Receiving"} = "boolean";
		$type{"ICMP Timestamp Broadcast Receiving"} = "boolean";
		$type{"ICMP Redirect Receiving"} = "antiboolean";
		$type{"ICMP Redirect Sending"} = "boolean";
		$type{"TCP Completed Connection Queue Size"} = "integer";
		$type{"TCP Connection Timeout"} = "millisecond";
		$type{"TCP Idle Timeout"} = "millisecond";
		$type{"TCP Keepalive Interval"} = "millisecond";
		$type{"TCP TIME_WAIT Timeout"} = "millisecond";
		$type{"TCP FIN_WAIT_2 Timeout"} = "millisecond";
		$type{"TCP ACK Delay Buffer Size"} = "millisecond";
		$type{"TCP Nagle Algorithm Buffer Size"} = "byte";
		$type{"TCP Sequence Number Generation Mode"} = "integer";
		$type{"TCP Extra Privileged Ports"} = "string";
		$type{"UDP Extra Privileged Ports"} = "string";

		# If this doesn't work on 2.5.1, it should be patched!
		if ($osver =~ /^5\.5\.1/) {
			$location{"TCP Incomplete Connection Queue Size"} = "/dev/tcp tcp_conn_req_max_q0";
			$location{"TCP Completed Connection Queue Size"} = "/dev/tcp tcp_conn_req_max_q";
			$location{"TCP Slow Start Mode"} = "/dev/tcp tcp_slow_start_initial";

			$type{"TCP Incomplete Connection Queue Size"} = "integer";
			$type{"TCP Completed Connection Queue Size"} = "integer";
			$type{"TCP Slow Start Mode"} = "integer";
		} elsif ($osver =~ /^5\.6/) {
			$location{"TCP Incomplete Connection Queue Size"} = "/dev/tcp tcp_conn_req_max_q0";
			$location{"TCP Completed Connection Queue Size"} = "/dev/tcp tcp_conn_req_max_q";
			$location{"TCP ACK Delay Queue Size"} = "/dev/tcp tcp_deferred_acks_max";
			$location{"TCP Slow Start Mode"} = "/dev/tcp tcp_slow_start_initial";
			$location{"TCP Slow Start After Idle Mode"} = "/dev/tcp tcp_slow_start_after_idle";
			$location{"TCP RST Rate Limit Mode"} = "/dev/tcp tcp_rst_sent_rate_enabled";
			$location{"TCP RST Rate Limit"} = "/dev/tcp tcp_rst_sent_rate";

			$type{"TCP Incomplete Connection Queue Size"} = "integer";
			$type{"TCP Completed Connection Queue Size"} = "integer";
			$type{"TCP ACK Delay Queue Size"} = "integer";
			$type{"TCP Slow Start Mode"} = "integer";
			$type{"TCP Slow Start After Idle Mode"} = "integer";
			$type{"TCP RST Rate Limit Mode"} = "boolean";
			$type{"TCP RST Rate Limit"} = "persecond";
		} elsif ($osver =~ /^5\.7/) {
			$location{"TCP Incomplete Connection Queue Size"} = "/dev/tcp tcp_conn_req_max_q0";
			$location{"TCP Completed Connection Queue Size"} = "/dev/tcp tcp_conn_req_max_q";
			$location{"TCP TIME_WAIT Timeout"} = "/dev/tcp tcp_time_wait_interval";
			$location{"TCP ACK Delay Queue Size"} = "/dev/tcp tcp_deferred_acks_max";
			$location{"TCP Slow Start Mode"} = "/dev/tcp tcp_slow_start_initial";
			$location{"TCP Slow Start After Idle Mode"} = "/dev/tcp tcp_slow_start_after_idle";
			$location{"TCP RST Rate Limit Mode"} = "/dev/tcp tcp_rst_sent_rate_enabled";
			$location{"TCP RST Rate Limit"} = "/dev/tcp tcp_rst_sent_rate";

			$type{"TCP Incomplete Connection Queue Size"} = "integer";
			$type{"TCP Completed Connection Queue Size"} = "integer";
			$type{"TCP TIME_WAIT Timeout"} = "millisecond";
			$type{"TCP ACK Delay Queue Size"} = "integer";
			$type{"TCP Slow Start Mode"} = "integer";
			$type{"TCP Slow Start After Idle Mode"} = "integer";
			$type{"TCP RST Rate Limit Mode"} = "boolean";
			$type{"TCP RST Rate Limit"} = "persecond";
		} elsif ($osver =~ /^5\.8/) {
			$location{"IPv6 Packet Forwarding"} = "/dev/ip ip6_forwarding";
			$location{"IPv6 Source-routed Packet Forwarding"} = "/dev/ip ip6_forward_src_routed";
			$location{"IPv6 Strict Multihoming"} = "/dev/ip ip6_strict_dst_multihoming";
			$location{"ICMPv6 Echo Multicast Receiving"} = "/dev/ip ip6_respond_to_echo_multicast";
			$location{"ICMPv6 Redirect Receiving"} = "/dev/ip ip6_ignore_redirect";
			$location{"ICMPv6 Redirect Sending"} = "/dev/ip ip6_send_redirects";
			$location{"TCP Incomplete Connection Queue Size"} = "/dev/tcp tcp_conn_req_max_q0";
			$location{"TCP Completed Connection Queue Size"} = "/dev/tcp tcp_conn_req_max_q";
			$location{"TCP TIME_WAIT Timeout"} = "/dev/tcp tcp_time_wait_interval";
			$location{"TCP ACK Delay Queue Size"} = "/dev/tcp tcp_deferred_acks_max";
			$location{"TCP Slow Start Mode"} = "/dev/tcp tcp_slow_start_initial";
			$location{"TCP Slow Start After Idle Mode"} = "/dev/tcp tcp_slow_start_after_idle";
			$location{"TCP RST Rate Limit Mode"} = "/dev/tcp tcp_rst_sent_rate_enabled";
			$location{"TCP RST Rate Limit"} = "/dev/tcp tcp_rst_sent_rate";

			$type{"IPv6 Packet Forwarding"} = "boolean";
			$type{"IPv6 Source-routed Packet Forwarding"} = "boolean";
			$type{"IPv6 Strict Multihoming"} = "boolean";
			$type{"ICMPv6 Echo Multicast Receiving"} = "boolean";
			$type{"ICMPv6 Redirect Receiving"} = "antiboolean";
			$type{"ICMPv6 Redirect Sending"} = "boolean";
			$type{"TCP Incomplete Connection Queue Size"} = "integer";
			$type{"TCP Completed Connection Queue Size"} = "integer";
			$type{"TCP TIME_WAIT Timeout"} = "millisecond";
			$type{"TCP ACK Delay Queue Size"} = "integer";
			$type{"TCP Slow Start Mode"} = "integer";
			$type{"TCP Slow Start After Idle Mode"} = "integer";
			$type{"TCP RST Rate Limit Mode"} = "boolean";
			$type{"TCP RST Rate Limit"} = "persecond";
		} elsif ($osver =~ /^5\.(9|10)/) {
			$location{"IPv6 Packet Forwarding"} = "/dev/ip ip6_forwarding";
			$location{"IPv6 Source-routed Packet Forwarding"} = "/dev/ip ip6_forward_src_routed";
			$location{"IPv6 Strict Multihoming"} = "/dev/ip ip6_strict_dst_multihoming";
			$location{"ICMPv6 Echo Multicast Receiving"} = "/dev/ip ip6_respond_to_echo_multicast";
			$location{"ICMPv6 Redirect Receiving"} = "/dev/ip ip6_ignore_redirect";
			$location{"ICMPv6 Redirect Sending"} = "/dev/ip ip6_send_redirects";
			$location{"TCP Incomplete Connection Queue Size"} = "/dev/tcp tcp_conn_req_max_q0";
			$location{"TCP Completed Connection Queue Size"} = "/dev/tcp tcp_conn_req_max_q";
			$location{"TCP TIME_WAIT Timeout"} = "/dev/tcp tcp_time_wait_interval";
			$location{"TCP ACK Delay Queue Size"} = "/dev/tcp tcp_deferred_acks_max";
			$location{"TCP ACK Delay LAN Buffer Size"} = "/dev/tcp tcp_local_dack_interval";
			$location{"TCP ACK Delay LAN Queue Size"} = "/dev/tcp tcp_local_dacks_max";
			$location{"TCP Slow Start Mode"} = "/dev/tcp tcp_slow_start_initial";
			$location{"TCP Slow Start After Idle Mode"} = "/dev/tcp tcp_slow_start_after_idle";
			$location{"TCP RST Rate Limit Mode"} = "/dev/tcp tcp_rst_sent_rate_enabled";
			$location{"TCP RST Rate Limit"} = "/dev/tcp tcp_rst_sent_rate";

			$type{"IPv6 Packet Forwarding"} = "boolean";
			$type{"IPv6 Source-routed Packet Forwarding"} = "boolean";
			$type{"IPv6 Strict Multihoming"} = "boolean";
			$type{"ICMPv6 Echo Multicast Receiving"} = "boolean";
			$type{"ICMPv6 Redirect Receiving"} = "antiboolean";
			$type{"ICMPv6 Redirect Sending"} = "boolean";
			$type{"TCP Incomplete Connection Queue Size"} = "integer";
			$type{"TCP Completed Connection Queue Size"} = "integer";
			$type{"TCP TIME_WAIT Timeout"} = "millisecond";
			$type{"TCP ACK Delay Queue Size"} = "integer";
			$type{"TCP ACK Delay LAN Buffer Size"} = "millisecond";
			$type{"TCP ACK Delay LAN Queue Size"} = "integer";
			$type{"TCP Slow Start Mode"} = "integer";
			$type{"TCP Slow Start After Idle Mode"} = "integer";
			$type{"TCP RST Rate Limit Mode"} = "boolean";
			$type{"TCP RST Rate Limit"} = "persecond";
		}

	# Linux (sysctl interface)
	# When does sysctl appear in Linux? Kernel 2.2 assumed.
	} elsif ($osname eq "Linux" && $osver =~ /^2\.[2-9]/) {
		$show = "/sbin/sysctl";

		$location{"IP Packet Forwarding"} = "net.ipv4.ip_forward";
		$location{"IP Source-routed Packet Receiving"} = "net.ipv4.conf.all.accept_source_route";
		$location{"IP Path MTU Discovery"} = "net.ipv4.ip_no_pmtu_disc";
		$location{"IP Local Port Range"} = "net.ipv4.ip_local_port_range";
		$location{"ICMP Echo Broadcast Receiving"} = "net.ipv4.icmp_echo_ignore_broadcasts";
		$location{"ICMP Redirect Receiving"} = "net.ipv4.conf.all.accept_redirects";
		$location{"ICMP Redirect Sending"} = "net.ipv4.conf.all.send_redirects";
		$location{"TCP Incomplete Connection Queue Size"} = "net.ipv4.tcp_max_syn_backlog";
		$location{"TCP Keepalive Interval"} = "net.ipv4.tcp_keepalive_time";
		$location{"TCP FIN_WAIT_2 Timeout"} = "net.ipv4.tcp_fin_timeout";
		$location{"TCP SYN Flood Protection"} = "net.ipv4.tcp_syncookies";

		$type{"IP Packet Forwarding"} = "boolean";
		$type{"IP Source-routed Packet Receiving"} = "boolean";
		$type{"IP Path MTU Discovery"} = "antiboolean";
		$type{"IP Local Port Range"} = "string";
		$type{"ICMP Echo Broadcast Receiving"} = "antiboolean";
		$type{"ICMP Redirect Receiving"} = "boolean";
		$type{"ICMP Redirect Sending"} = "boolean";
		$type{"TCP Incomplete Connection Queue Size"} = "integer";
		$type{"TCP Keepalive Interval"} = "second";
		$type{"TCP FIN_WAIT_2 Timeout"} = "second";
		$type{"TCP SYN Flood Protection"} = "boolean";

	# Linux (/proc filesystem)
	} elsif ($osname eq "Linux" && $osver =~ /^2\.0\./) {
		$show = "/bin/cat";

		$location{"IP Packet Forwarding"} = "/proc/sys/net/ipv4/ip_forward";
		$location{"IP Source-routed Packet Receiving"} = "/proc/sys/net/ipv4/conf/all/accept_source_route";
		$location{"IP Path MTU Discovery"} = "/proc/sys/net/ipv4/ip_no_pmtu_disc";
		$location{"IP Local Port Range"} = "/proc/sys/net/ipv4/ip_local_port_range";
		$location{"ICMP Echo Broadcast Receiving"} = "/proc/sys/net/ipv4/icmp_echo_ignore_broadcasts";
		$location{"ICMP Redirect Receiving"} = "/proc/sys/net/ipv4/conf/all/accept_redirects";
		$location{"ICMP Redirect Sending"} = "/proc/sys/net/ipv4/conf/all/send_redirects";
		$location{"TCP Incomplete Connection Queue Size"} = "/proc/sys/net/ipv4/tcp_max_syn_backlog";
		$location{"TCP Keepalive Interval"} = "/proc/sys/net/ipv4/tcp_keepalive_time";
		$location{"TCP FIN_WAIT_2 Timeout"} = "/proc/sys/net/ipv4/tcp_fin_timeout";
		$location{"TCP SYN Flood Protection"} = "/proc/sys/net/ipv4/tcp_syncookies";

		$type{"IP Packet Forwarding"} = "boolean";
		$type{"IP Source-routed Packet Receiving"} = "boolean";
		$type{"IP Path MTU Discovery"} = "antiboolean";
		$type{"IP Local Port Range"} = "string";
		$type{"ICMP Echo Broadcast Receiving"} = "antiboolean";
		$type{"ICMP Redirect Receiving"} = "boolean";
		$type{"ICMP Redirect Sending"} = "boolean";
		$type{"TCP Incomplete Connection Queue Size"} = "integer";
		$type{"TCP Keepalive Interval"} = "second";
		$type{"TCP FIN_WAIT_2 Timeout"} = "second";
		$type{"TCP SYN Flood Protection"} = "boolean";

	# HP-UX 10.x
	} elsif ($osname eq "HP-UX" && $osver =~ /^10\./) {
		$show = "/usr/contrib/bin/nettune -l";

		$location{"IP Packet Forwarding"} = "ip_forwarding";
		$location{"IP Directed Broadcast Packet Forwarding"} = "ip_forward_directed_broadcasts";
		$location{"TCP Incomplete Connection Queue Size"} = "so_qlimit_max";
		$location{"TCP Completed Connection Queue Size"} = "ip_intrqmax";
		$location{"TCP Keepalive Interval"} = "tcp_keepstart";
		$location{"TCP SYN Flood Protection"} = "hp_syn_protect";
		$location{"TCP Sequence Number Generation Mode"} = "tcp_random_seq";

		$type{"IP Packet Forwarding"} = "boolean";
		$type{"IP Directed Broadcast Packet Forwarding"} = "boolean";
		$type{"TCP Incomplete Connection Queue Size"} = "integer";
		$type{"TCP Completed Connection Queue Size"} = "integer";
		$type{"TCP Keepalive Interval"} = "second";
		$type{"TCP SYN Flood Protection"} = "boolean";
		$type{"TCP Sequence Number Generation Mode"} = "integer";

	# HP-UX 11.x
	} elsif ($osname eq "HP-UX" && $osver =~ /^11\./) {
		$show = "/usr/sbin/ndd";

		$location{"IP Packet Forwarding"} = "/dev/ip ip_forwarding";
		$location{"IP Directed Broadcast Packet Forwarding"} = "/dev/ip ip_forward_directed_broadcasts";
		$location{"IP Source-routed Packet Forwarding"} = "/dev/ip ip_forward_src_routed";
		$location{"IP Path MTU Discovery"} = "/dev/ip ip_pmtu_strategy";
		$location{"ICMP Echo Broadcast Receiving"} = "/dev/ip ip_respond_to_echo_broadcast";
		$location{"ICMP Timestamp Broadcast Receiving"} = "/dev/ip ip_respond_to_timestamp_broadcast";
		$location{"ICMP Redirect Sending"} = "/dev/ip ip_send_redirects";
		$location{"TCP Incomplete Connection Queue Size"} = "/dev/tcp tcp_syn_rcvd_max";
		$location{"TCP Completed Connection Queue Size"} = "/dev/tcp tcp_conn_request_max";
		$location{"TCP Connection Timeout"} = "/dev/tcp tcp_ip_abort_cinterval";
		$location{"TCP Keepalive Interval"} = "/dev/tcp tcp_keepalive_interval";
		$location{"TCP TIME_WAIT Timeout"} = "/dev/tcp tcp_time_wait_interval";
		$location{"TCP FIN_WAIT_2 Keepalive Interval"} = "/dev/tcp tcp_keepalive_detached_interval";
		$location{"TCP ACK Delay Buffer Size"} = "/dev/tcp tcp_deferred_ack_interval";
		$location{"TCP Nagle Algorithm Buffer Size"} = "/dev/tcp tcp_naglim_def";

		$type{"IP Packet Forwarding"} = "boolean";
		$type{"IP Directed Broadcast Packet Forwarding"} = "boolean";
		$type{"IP Source-routed Packet Forwarding"} = "boolean";
		$type{"IP Path MTU Discovery"} = "boolean";
		$type{"ICMP Echo Broadcast Receiving"} = "boolean";
		$type{"ICMP Timestamp Broadcast Receiving"} = "boolean";
		$type{"ICMP Redirect Sending"} = "boolean";
		$type{"TCP Incomplete Connection Queue Size"} = "integer";
		$type{"TCP Completed Connection Queue Size"} = "integer";
		$type{"TCP Connection Timeout"} = "millisecond";
		$type{"TCP Keepalive Interval"} = "millisecond";
		$type{"TCP TIME_WAIT Timeout"} = "millisecond";
		$type{"TCP FIN_WAIT_2 Keepalive Interval"} = "millisecond";
		$type{"TCP ACK Delay Buffer Size"} = "byte";
		$type{"TCP Nagle Algorithm Buffer Size"} = "byte";

	# AIX
	} elsif ($osname eq "AIX") {
		# Assume that if OS version number doesn't include a decimal
		# point, that it's a version of AIX 5L. (Why does IBM do stuff
		# like this?)
		$osver !~ /\./ and $osver = "5.$osver";

		$show = "/usr/sbin/no -o";

		$location{"IP Packet Forwarding"} = "ipforwarding";
		$location{"IP Directed Broadcast Packet Forwarding"} = "directed_broadcast";
		$location{"IP Source-routed Packet Non-local Sending"} = "nonlocsrcroute";
		$location{"IPv6 Packet Forwarding"} = "ip6forwarding";
		$location{"IPv6 Source-routed Packet Forwarding"} = "ip6srcrouteforward";
		$location{"ICMP Echo Broadcast Receiving"} = "bcastping";
		$location{"ICMP Redirect Sending"} = "ipsendredirects";
		$location{"TCP Completed Connection Queue Size"} = "ipqmaxlen";
		$location{"TCP Connection Timeout"} = "tcp_keepinit";
		$location{"TCP Idle Timeout"} = "tcp_keepidle";
		$location{"TCP Keepalive Interval"} = "tcp_keepintvl";
		$location{"TCP TIME_WAIT Timeout"} = "tcp_timewait";
		$location{"TCP SYN Flood Protection"} = "clean_partial_conns";

		$type{"IP Packet Forwarding"} = "boolean";
		$type{"IP Directed Broadcast Packet Forwarding"} = "boolean";
		$type{"IP Source-routed Packet Non-local Sending"} = "boolean";
		$type{"IPv6 Packet Forwarding"} = "boolean";
		$type{"IPv6 Source-routed Packet Forwarding"} = "boolean";
		$type{"ICMP Echo Broadcast Receiving"} = "boolean";
		$type{"ICMP Redirect Sending"} = "boolean";
		$type{"TCP Completed Connection Queue Size"} = "integer";
		$type{"TCP Connection Timeout"} = "halfsecond";
		$type{"TCP Idle Timeout"} = "halfsecond";
		$type{"TCP Keepalive Interval"} = "halfsecond";
		$type{"TCP TIME_WAIT Timeout"} = "quarterminute";
		$type{"TCP SYN Flood Protection"} = "boolean";

		if ($osver =~ /^4\.1/) {
			$location{"TCP Incomplete Connection Queue Size"} = "somaxconn";

			$type{"TCP Incomplete Connection Queue Size"} = "integer";
		} elsif ($osver =~ /^4\.2/) {
			$location{"IP Source-routed Packet Forwarding"} = "ipsrcrouteforward";
			$location{"IP Source-routed Packet Receiving"} = "ipsrcrouterecv";
			$location{"IP Source-routed Packet Sending"} = "ipsrcroutesend";
			$location{"ICMP Redirect Receiving"} = "ipignoreredirects";
			$location{"TCP Incomplete Connection Queue Size"} = "somaxconn";
			$location{"TCP Path MTU Discovery"} = "tcp_pmtu_discover";
			$location{"UDP Path MTU Discovery"} = "udp_pmtu_discover";

			$type{"IP Source-routed Packet Forwarding"} = "boolean";
			$type{"IP Source-routed Packet Receiving"} = "boolean";
			$type{"IP Source-routed Packet Sending"} = "boolean";
			$type{"ICMP Redirect Receiving"} = "antiboolean";
			$type{"TCP Incomplete Connection Queue Size"} = "integer";
			$type{"TCP Path MTU Discovery"} = "boolean";
			$type{"UDP Path MTU Discovery"} = "boolean";
		} elsif ($osver =~ /^4\.3/) {
			$location{"IP Source-routed Packet Forwarding"} = "ipsrcrouteforward";
			$location{"IP Source-routed Packet Receiving"} = "ipsrcrouterecv";
			$location{"IP Source-routed Packet Sending"} = "ipsrcroutesend";
			$location{"ICMP Redirect Receiving"} = "ipignoreredirects";
			$location{"TCP Incomplete Connection Queue Size"} = "somaxconn";
			$location{"TCP ACK Delay"} = "delayack";
			$location{"TCP Path MTU Discovery"} = "tcp_pmtu_discover";
			$location{"UDP Path MTU Discovery"} = "udp_pmtu_discover";

			$type{"IP Source-routed Packet Forwarding"} = "boolean";
			$type{"IP Source-routed Packet Receiving"} = "boolean";
			$type{"IP Source-routed Packet Sending"} = "boolean";
			$type{"ICMP Redirect Receiving"} = "antiboolean";
			$type{"TCP Incomplete Connection Queue Size"} = "integer";
			$type{"TCP ACK Delay"} = "integer";
			$type{"TCP Path MTU Discovery"} = "boolean";
			$type{"UDP Path MTU Discovery"} = "boolean";
		} elsif ($osver =~ /^5\./) {
			$location{"IP Source-routed Packet Forwarding"} = "ipsrcrouteforward";
			$location{"IP Source-routed Packet Receiving"} = "ipsrcrouterecv";
			$location{"IP Source-routed Packet Sending"} = "ipsrcroutesend";
			$location{"IPv6 Strict Multihoming"} = "multi_homed";
			$location{"ICMP Redirect Receiving"} = "ipignoreredirects";
			$location{"TCP Incomplete Connection Queue Size"} = "somaxconn";
			$location{"TCP FIN_WAIT_2 Timeout"} = "tcp_finwait2";
			$location{"TCP ACK Delay"} = "tcp_nodelayack";
			$location{"TCP Nagle Algorithm Buffer Size"} = "tcp_nagle_limit";
			$location{"TCP Blind Attack Protection Mode"} = "tcp_tcpsecure";
			$location{"TCP RST Rate Limit Mode"} = "tcp_bad_port_limit";
			$location{"TCP RST Rate Limit"} = "tcp_bad_port_limit";
			$location{"UDP ICMP Error Rate Limit Mode"} = "udp_bad_port_limit";
			$location{"UDP ICMP Error Rate Limit"} = "udp_bad_port_limit";
			$location{"TCP Path MTU Discovery"} = "tcp_pmtu_discover";
			$location{"UDP Path MTU Discovery"} = "udp_pmtu_discover";

			$type{"IP Source-routed Packet Forwarding"} = "boolean";
			$type{"IP Source-routed Packet Receiving"} = "boolean";
			$type{"IP Source-routed Packet Sending"} = "boolean";
			$type{"IPv6 Strict Multihoming"} = "integer";
			$type{"ICMP Redirect Receiving"} = "antiboolean";
			$type{"TCP Incomplete Connection Queue Size"} = "integer";
			$type{"TCP FIN_WAIT_2 Timeout"} = "halfsecond";
			$type{"TCP ACK Delay"} = "antiboolean";
			$type{"TCP Nagle Algorithm Buffer Size"} = "byte";
			$type{"TCP Blind Attack Protection Mode"} = "integer";
			$type{"TCP RST Rate Limit Mode"} = "boolean";
			$type{"TCP RST Rate Limit"} = "perhalfsecond";
			$type{"UDP ICMP Error Rate Limit Mode"} = "boolean";
			$type{"UDP ICMP Error Rate Limit"} = "perhalfsecond";
			$type{"TCP Path MTU Discovery"} = "boolean";
			$type{"UDP Path MTU Discovery"} = "boolean";
		}

	# *BSD
	} elsif ($osname =~ /BSD$/) {
		$show = "/sbin/sysctl -n";

		$location{"IP Packet Forwarding"} = "net.inet.ip.forwarding";
		$location{"IP Source-routed Packet Forwarding"} = "net.inet.ip.sourceroute";
		$location{"IP Source-routed Packet Receiving"} = "net.inet.ip.accept_sourceroute";
		$location{"IP Strict Multihoming"} = "net.inet.ip.check_interface";
		$location{"IPv6 Packet Forwarding"} = "net.inet6.ip6.forwarding";
		$location{"ICMP Echo Broadcast Receiving"} = "net.inet.icmp.bmcastecho";
		$location{"ICMP Redirect Receiving"} = "net.inet.icmp.drop_redirect";
		$location{"ICMP Redirect Sending"} = "net.inet.ip.redirect";
		$location{"ICMPv6 Redirect Sending"} = "net.inet6.ip6.redirect";
		$location{"TCP Incomplete Connection Queue Size"} = "kern.ipc.somaxconn";

		$type{"IP Packet Forwarding"} = "boolean";
		$type{"IP Source-routed Packet Forwarding"} = "boolean";
		$type{"IP Source-routed Packet Receiving"} = "boolean";
		$type{"IP Strict Multihoming"} = "boolean";
		$type{"IPv6 Packet Forwarding"} = "boolean";
		$type{"ICMP Echo Broadcast Receiving"} = "boolean";
		$type{"ICMP Redirect Receiving"} = "antiboolean";
		$type{"ICMP Redirect Sending"} = "boolean";
		$type{"ICMPv6 Redirect Sending"} = "boolean";
		$type{"TCP Incomplete Connection Queue Size"} = "integer";

	# Mac OS X, Darwin
	} elsif ($osname eq "Darwin") {
		$show = "/usr/sbin/sysctl -n";

		# A few of these have not yet been implemented in Darwin.
		$location{"IP Packet Forwarding"} = "net.inet.ip.forwarding";
		$location{"IP Source-routed Packet Forwarding"} = "net.inet.ip.sourceroute";
		$location{"IP Source-routed Packet Receiving"} = "net.inet.ip.accept_sourceroute";
		$location{"IP Strict Multihoming"} = "net.inet.ip.check_interface";
		$location{"IP Path MTU Discovery"} = "net.inet.tcp.path_mtu_discovery";
		$location{"IPv6 Packet Forwarding"} = "net.inet6.ip6.forwarding";
		$location{"ICMP Echo Broadcast Receiving"} = "net.inet.icmp.bmcastecho";
		$location{"ICMP Timestamp Receiving"} = "net.inet.icmp.timestamp";
		$location{"ICMP Redirect Receiving"} = "net.inet.icmp.drop_redirect";
		$location{"ICMP Redirect Sending"} = "net.inet.ip.redirect";
		$location{"ICMPv6 Redirect Receiving"} = "net.inet6.icmp6.rediraccept";
		$location{"ICMPv6 Redirect Sending"} = "net.inet6.ip6.redirect";
		$location{"TCP Incomplete Connection Queue Size"} = "kern.ipc.somaxconn";
		$location{"TCP Completed Connection Queue Size"} = "net.inet.ip.intr_queue_maxlen";
		$location{"TCP Connection Timeout"} = "net.inet.tcp.keepinit";
		$location{"TCP Idle Timeout"} = "net.inet.tcp.keepidle";
		$location{"TCP Keepalive Interval"} = "net.inet.tcp.keepintvl";
		$location{"TCP ACK Delay"} = "net.inet.tcp.delayed_ack";
		$location{"TCP Slow Start Mode"} = "net.inet.tcp.slowstart_flightsize";
		$location{"TCP Sequence Number Generation Mode"} = "net.inet.tcp.strict_rfc1948";

		$type{"IP Packet Forwarding"} = "boolean";
		$type{"IP Source-routed Packet Forwarding"} = "boolean";
		$type{"IP Source-routed Packet Receiving"} = "boolean";
		$type{"IP Strict Multihoming"} = "boolean";
		$type{"IP Path MTU Discovery"} = "boolean";
		$type{"IPv6 Packet Forwarding"} = "boolean";
		$type{"ICMP Echo Broadcast Receiving"} = "boolean";
		$type{"ICMP Timestamp Receiving"} = "boolean";
		$type{"ICMP Redirect Receiving"} = "antiboolean";
		$type{"ICMP Redirect Sending"} = "boolean";
		$type{"ICMPv6 Redirect Receiving"} = "boolean";
		$type{"ICMPv6 Redirect Sending"} = "boolean";
		$type{"TCP Incomplete Connection Queue Size"} = "integer";
		$type{"TCP Completed Connection Queue Size"} = "integer";
		$type{"TCP Connection Timeout"} = "second";
		$type{"TCP Idle Timeout"} = "second";
		$type{"TCP Keepalive Interval"} = "second";
		$type{"TCP ACK Delay"} = "boolean";
		$type{"TCP Slow Start Mode"} = "boolean";
		$type{"TCP Sequence Number Generation Mode"} = "boolean";

	} else {
		die "$progname: Unsupported operating system: $osname $osver\n";
	}
}

sub showParams {
	my $value;

	my ($longestParamName) = reverse sort byLength @parameter;
	my $paramFieldLen = (length $longestParamName) + 2;

	foreach (@parameter) {
		if ($location{$_}) {
			$value = `$show $location{$_}`;
			chomp $value;
			# AIX strikes again!
			# Linux, too!
			$value =~ /^[\w\.]+ = (.+)$/ and $value = $+;

			if ($type{$_} eq "boolean") {
				$value = ($value == 0) ? "off" : "on";
			} elsif ($type{$_} eq "antiboolean") {
				$value = ($value == 0) ? "on" : "off";
				$value = $value ? "off" : "on";
			} elsif ($type{$_} eq "byte") {
				if ($value =~ /^\d+$/) {
					$value = "$value bytes";
				} else {
					$value = "improper value";
				}
			} elsif ($type{$_} eq "kilobyte") {
				if ($value =~ /^\d+$/) {
					$value *= 1024;
					$value = "$value bytes";
				} else {
					$value = "improper value";
				}
			} elsif ($type{$_} eq "block") {
				if ($value =~ /^\d+$/) {
					$value *= 512;
					$value *= 1024;
					$value = "$value bytes";
				} else {
					$value = "improper value";
				}
			} elsif ($type{$_} eq "megabyte") {
				if ($value =~ /^\d+$/) {
					$value *= 1024;
					$value *= 1024;
					$value = "$value bytes";
				} else {
					$value = "improper value";
				}
			} elsif ($type{$_} eq "integer") {
				$value =~ /^\-?\d+$/ or
					$value = "improper value";
			} elsif ($type{$_} eq "millisecond") {
				if ($value =~ /^\d+$/) {
					$value /= 1000;
					$value = sprintf "%0.3f", $value;
					$value =~ s/0*$//;
					$value =~ s/\.$//;
					$value = "$value seconds";
				} else {
					$value = "improper value";
				}
			} elsif ($type{$_} eq "halfsecond") {
				if ($value =~ /^\d+$/) {
					$value /= 2;
					$value = sprintf "%0.1f", $value;
					$value =~ s/0*$//;
					$value =~ s/\.$//;
					$value = "$value seconds";
				} else {
					$value = "improper value";
				}
			} elsif ($type{$_} eq "perhalfsecond") {
				if ($value =~ /^\d+$/) {
					$value *= 2;
					$value = "$value per second";
				} else {
					$value = "improper value";
				}
			} elsif ($type{$_} eq "second") {
				if ($value =~ /^\d+$/) {
					$value = "$value seconds";
				} else {
					$value = "improper value";
				}
			} elsif ($type{$_} eq "per second") {
				if ($value =~ /^\d+$/) {
					$value = "$value per second";
				} else {
					$value = "improper value";
				}
			} elsif ($type{$_} eq "quarterminute") {
				if ($value =~ /^\d+$/) {
					$value *= 15;
					$value = "$value seconds";
				} else {
					$value = "improper value";
				}
			} elsif ($type{$_} eq "minute") {
				if ($value =~ /^\d+$/) {
					$value *= 60;
					$value = "$value seconds";
				} else {
					$value = "improper value";
				}
			} elsif ($type{$_} eq "string") {
				$value =~ s/\n/ /g;
			}
			printf "%-${paramFieldLen}s%15s\n", "$_:", $value;
		}
	}
}

sub byLength {
	length $a <=> length $b;
}
